/*
 * Copyright (C) 2002 Richard Moore <rich@kde.org>
 *               2003 Dirk Ziegelmeier <dziegel@gmx.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <klocale.h>
#include <klibloader.h>
#include <kconfig.h>

#include <qspinbox.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qtimer.h>

#include "qtvisionwidget.h"
#include "qtvision.h"

#include "screensaver.h"
#include "screensaver.moc"

// libkscreensaver interface
extern "C"
{
    const char *kss_applicationName = "qtvisionsaver.kss";
    const char *kss_description = I18N_NOOP( "A screensaver that displays a TV screen" );
    const char *kss_version = "0.1";

    KScreenSaver *kss_create( WId id )
    {
        KLibLoader::self()->globalLibrary("libqtvision.so");
        return new QtVisionSaver( id );
    }

    QDialog *kss_setup()
    {
        return new QtVisionSaverSetup(0L, "QtVision Screensaver Setup");
    }
}

QtVisionSaver::QtVisionSaver( WId id )
    : KScreenSaver( id )
{
    _screen = new QtVisionWidget((QWidget*)0L);
    embed(_screen);
    _screen->show();
    _screen->driver()->start();

    QTimer::singleShot(500, this, SLOT(applySettings()));
}

QtVisionSaver::~QtVisionSaver()
{
    delete _screen;
}

void QtVisionSaver::applySettings()
{
    KConfig *config = KGlobal::config();
    config->setGroup("Settings");
    _screen->driver()->setChannel(config->readNumEntry("Channel", 2));
    _screen->driver()->setVolume(config->readNumEntry("Volume", 50));
}

//----------------------------------------------

QtVisionSaverSetup::QtVisionSaverSetup( QWidget *parent, const char *name )
	: KDialogBase( parent, name, true, i18n( "QtVision Screen Saver Setup" ),
	  Ok|Cancel, Ok, true )
{
    QWidget* main = makeMainWidget();
    _grid = new QGridLayout(main, 3, 2);

    _channelLabel = new QLabel(i18n("Channel:"), main);
    _grid->addWidget(_channelLabel, 0, 0);
    _channelSpinbox = new QSpinBox(1, 200, 1, main);
    _grid->addWidget(_channelSpinbox, 0, 1);

    _volumeLabel = new QLabel(i18n("Volume:"), main);
    _grid->addWidget(_volumeLabel, 1, 0);
    _volumeSpinbox = new QSpinBox(1, 100, 5, main);
    _grid->addWidget(_volumeSpinbox, 1, 1);

    _infoLabel = new QLabel(i18n("Info: to configure the other options,\nplease use the QtVision application!"), main);
    _infoLabel->setAlignment(AlignHCenter);
    _grid->addMultiCellWidget(_infoLabel, 2, 2, 0, 1);


    KConfig *config = KGlobal::config();
    config->setGroup("Settings");
    _channelSpinbox->setValue(config->readNumEntry("Channel", 2));
    _volumeSpinbox->setValue(config->readNumEntry("Volume", 50));
}

QtVisionSaverSetup::~QtVisionSaverSetup()
{
}

void QtVisionSaverSetup::slotOk()
{
    KConfig *config = KGlobal::config();
    config->setGroup("Settings");

    config->writeEntry("Channel", _channelSpinbox->value());
    config->writeEntry("Volume", _volumeSpinbox->value());
    config->sync();

    accept();
}
