/***************************************************************************
                          devicewidgetimpl.cpp  -  description
                             -------------------
    begin                : Mon Jun 3 2002
    copyright            : (C) 2002 by Kevin Hessels, George Staikos, Dirk Ziegelmeier
    email                : khessels@shaw.ca, staikos@kde.org, dziegel@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qcombobox.h>

#include <kdebug.h>
#include <klocale.h>
#include <kmessagebox.h>

#include "cfgdata.h"
#include "devicewidgetimpl.h"
#include "qtvision.h"
#include "sourcemanager.h"


DeviceWidgetImpl::DeviceWidgetImpl(QWidget *parent, QtVision *qtv, ConfigData* cfg, KConfig* h)
    : DeviceWidget(parent),
      SettingsDialogPage(i18n("Device"),
			 i18n("Configure device options here"),
			 "kcmpci"),
      _qtv(qtv),
      _srcm(qtv->sourceManager()),
      _cfg(cfg),
      _h(h)
{
    connect(_device, SIGNAL(highlighted(const QString &)),
	    this, SLOT(slotChangeSources(const QString &)));
} // DeviceWidgetImpl constructor


DeviceWidgetImpl::~DeviceWidgetImpl()
{
} // DeviceWidgetImpl destructor

void DeviceWidgetImpl::setup()
{
    _device->clear();
    _device->insertStringList(_srcm->deviceList());
    slotChangeSources(_device->currentText());

    if (_srcm->deviceList().isEmpty()) {
	KMessageBox::error(0L,
			   i18n("No devices found.\nFor more information, please read the FAQ at\nhttp://www.kwintv.org"),
			   i18n("Error: no devices found"));
    }

    // preselect current device
    if (_qtv->device()) {
        QString dt = _qtv->device()->device();
        for (int i = 0; i < _device->count(); i++) {
            if (_device->text(i) == dt) {
               _device->setCurrentItem(i);
               break;
            }
        }

        QString st = _qtv->device()->source();
        for (int i = 0; i < _source->count(); i++) {
            if (_source->text(i) == st) {
               _source->setCurrentItem(i);
               break;
            }
        }

        QString et = _qtv->device()->encoding();
        for (int i = 0; i < _encoding->count(); i++) {
            if (_encoding->text(i) == et) {
               _encoding->setCurrentItem(i);
               break;
            }
        }
    }
}

void DeviceWidgetImpl::apply()
{
    if ((_qtv->device()             == 0)                        ||
	(_qtv->device()->device()   != _device->currentText())   ||
	(_qtv->device()->source()   != _source->currentText())   ||
	(_qtv->device()->encoding() != _encoding->currentText())    ) {
	_qtv->setDevice(0L);
	QVSourcePlugin *vsrc = _srcm->pluginFor(_device->currentText());
	if (vsrc == 0) return;
	vsrc->setDevice(_device->currentText());
	_qtv->setDevice(vsrc);
	_qtv->setEncoding(_encoding->currentText());
	_qtv->setSource(_source->currentText());
	_qtv->startCapture();
    }
}

void DeviceWidgetImpl::defaults()
{
}

/***************************************************************************
 *
 * Protected Slots
 *
 ***************************************************************************/

void DeviceWidgetImpl::slotChangeSources(const QString &dev)
{
    _source->clear();
    _source->insertStringList(_srcm->sourcesFor(dev));
    QString lastSource = _cfg->lastSourceFor(dev);
    if (!lastSource.isEmpty()) {
        for (int i = 0; i < this->_source->count(); i++) {
            if (this->_source->text(i) == lastSource) {
                this->_source->setCurrentItem(i);
                break;
            }
        }
    }

    _encoding->clear();
    _encoding->insertStringList(_srcm->encodingsFor(dev));
    QString lastEncoding = _cfg->lastEncodingFor(dev);
    if (!lastEncoding.isEmpty()) {
        for (int i = 0; i < this->_encoding->count(); i++) {
            if (this->_encoding->text(i) == lastEncoding) {
                this->_encoding->setCurrentItem(i);
                break;
            }
        }
    }
} // slotChangeSources

#include "devicewidgetimpl.moc"
