/***************************************************************************
                           generalwidgetimpl.cpp
                           ---------------------
    begin                : Sun Sep 6 2003
    copyright            : (C) 2003 by Dirk Ziegelmeier
    email                : dziegel@gmx.de
    rcs data             : $Id: generalwidgetimpl.cpp,v 1.6 2003/10/22 10:32:31 dziegel Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <klocale.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <kfiledialog.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include "generalwidgetimpl.h"
#include "cfgdata.h"
#include "channelstore.h"
#include "sourcemanager.h"
#include "qtvision.h"

GeneralWidgetImpl::GeneralWidgetImpl(QWidget* parent, QtVision* qtv, ConfigData* cfg, KConfig* h)
    : GeneralWidget(parent),
      SettingsDialogPage(i18n("General"),
			 i18n("Configure general options here"),
			 "misc"),
      _qtv(qtv),
      _srcm(qtv->sourceManager()),
      _cfg(cfg),
      _h(h)
{
    connect(_edit, SIGNAL( clicked() ),
	    this, SLOT( slotEditClicked() ));
    connect(_fileList, SIGNAL( doubleClicked(QListViewItem*) ),
	    this, SLOT( slotEditClicked() ));
    connect(_fileList, SIGNAL( selectionChanged() ),
	    this, SLOT( slotSelectionChanged() ));
    _edit->setEnabled(false);
}

GeneralWidgetImpl::~GeneralWidgetImpl()
{
}

void GeneralWidgetImpl::setup()
{
    // _maxRes->setChecked(_cfg->maxResolution);
    _autoStart->setChecked(_cfg->autoStart);
    _useListingsURL->setChecked(_cfg->useListingsURL);
    _listingsURL->setText(_cfg->listingsURL);
    _channelNumberingStart->setValue(_cfg->channelNumberingStart);
    _videoDesktopEnabled->setChecked(_cfg->videoDesktopEnabled);
    _showSelected->setChecked(_cfg->showSelectedOnly);

    const QStringList& devs = _srcm->deviceList();
    _fileList->clear();
    
    for (QStringList::const_iterator it = devs.begin();
	 it != devs.end();
	 ++it) {
	_h->setGroup(*it);
	new QListViewItem(_fileList, *it, _h->readPathEntry("Channel File", _qtv->channels()->defaultFileName()));
    }
}

void GeneralWidgetImpl::defaults()
{
    _autoStart->setChecked(true);
    _useListingsURL->setChecked(true);
    _listingsURL->setText("http://www.ananova.com/tv/");
    _channelNumberingStart->setValue(2);
    _videoDesktopEnabled->setChecked(false);
    _showSelected->setChecked(true);

    const QStringList& devs = _srcm->deviceList();
    _fileList->clear();

    for (QStringList::const_iterator it = devs.begin();
	 it != devs.end();
	 ++it) {
	new QListViewItem(_fileList, *it, _qtv->channels()->defaultFileName());
    }
}

void GeneralWidgetImpl::apply()
{
    _cfg->autoStart             = _autoStart->isChecked();
    // _cfg->maxResolution         = _maxRes->isChecked();
    _cfg->videoDesktopEnabled   = _videoDesktopEnabled->isChecked();
    _cfg->useListingsURL        = _useListingsURL->isChecked();
    _cfg->listingsURL           = _listingsURL->text();
    _cfg->channelNumberingStart = _channelNumberingStart->value();
    _cfg->showSelectedOnly      = _showSelected->isChecked();

    bool changed = false;
    for (QListViewItemIterator it(_fileList);
	 *it != 0;
	 ++it) {
	_h->setGroup((*it)->text(0));
	if (_h->readPathEntry("Channel File", _qtv->channels()->defaultFileName()) != (*it)->text(1)) {
	    changed = true;
	}
    }

    if (changed) {
	_qtv->saveChannels();
	for (QListViewItemIterator it(_fileList);
	     *it != 0;
	     ++it) {
	    _h->setGroup((*it)->text(0));
	    _h->writePathEntry("Channel File", (*it)->text(1));
	    if ( (_qtv->device())                             &&
		 ((*it)->text(0) == _qtv->device()->device())    ) {
		_cfg->channelFile = (*it)->text(1);
	    }
	}
	_qtv->reloadChannels();
    }
}

void GeneralWidgetImpl::slotEditClicked()
{
    if (0L == _fileList->selectedItem()) return;

    QString fn = KFileDialog::getOpenFileName(KGlobal::dirs()->saveLocation("qtvision"),
					      QString("*"),
					      this,
					      i18n("Select Channel File"));
    if (!fn.isEmpty()) {
	_fileList->selectedItem()->setText(1, fn);
    }
}

void GeneralWidgetImpl::slotSelectionChanged()
{
    _edit->setEnabled(0L != _fileList->selectedItem());
}

#include "generalwidgetimpl.moc"
