/***************************************************************************
                           picturewidgetimpl.cpp
                           -------------------
    begin                : Sun Sep 6 2003
    copyright            : (C) 2003 by Dirk Ziegelmeier
    email                : dziegel@gmx.de
    rcs data             : $Id: picturewidgetimpl.cpp,v 1.5 2003/10/22 10:32:31 dziegel Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <klocale.h>
#include "picturewidgetimpl.h"
#include <qslider.h>
#include <qcheckbox.h>
#include "cfgdata.h"
#include "qtvision.h"

PictureWidgetImpl::PictureWidgetImpl(QWidget* parent, QtVision* qtv, ConfigData* cfg, KConfig* h)
    : PictureWidget(parent),
      SettingsDialogPage(i18n("Picture"),
			 i18n("Configure picture options here"),
			 "colorize"),
      _qtv(qtv),
      _cfg(cfg),
      _h(h)
{
}

PictureWidgetImpl::~PictureWidgetImpl()
{
}

void PictureWidgetImpl::setup()
{
    connect(_hue, SIGNAL(valueChanged(int)),
	    this, SLOT(slotHueChanged(int)));
    connect(_colour, SIGNAL(valueChanged(int)),
	    this, SLOT(slotColourChanged(int)));
    connect(_brightness, SIGNAL(valueChanged(int)),
	    this, SLOT(slotBrightnessChanged(int)));
    connect(_whiteness, SIGNAL(valueChanged(int)),
	    this, SLOT(slotWhitenessChanged(int)));
    connect(_contrast, SIGNAL(valueChanged(int)),
	    this, SLOT(slotContrastChanged(int)));

    _oldHue = _qtv->hue();
    _hue->setValue(_qtv->hue());

    _oldColour = _qtv->colour();
    _colour->setValue(_qtv->colour());

    _oldBrightness = _qtv->brightness();
    _brightness->setValue(_qtv->brightness());

    _oldWhiteness = _qtv->whiteness();
    _whiteness->setValue(_qtv->whiteness());

    _oldContrast = _qtv->contrast();
    _contrast->setValue(_qtv->contrast());

    _autoScale->setChecked(_cfg->autoScaleEnabled);
}

void PictureWidgetImpl::defaults()
{
    slotHueChanged(32768);
    slotColourChanged(32768);
    slotBrightnessChanged(32768);
    slotWhitenessChanged(32768);
    slotContrastChanged(32768);

    _autoScale->setChecked(true);

    setup();
}

void PictureWidgetImpl::apply()
{
    _cfg->hue              = _hue->value();
    _cfg->colour           = _colour->value();
    _cfg->brightness       = _brightness->value();
    _cfg->whiteness        = _whiteness->value();
    _cfg->contrast         = _contrast->value();
    _cfg->autoScaleEnabled = _autoScale->isChecked();
}

void PictureWidgetImpl::cancel()
{
    slotHueChanged(_oldHue);
    slotColourChanged(_oldColour);
    slotBrightnessChanged(_oldBrightness);
    slotWhitenessChanged(_oldWhiteness);
    slotContrastChanged(_oldContrast);
}

/***************************************************************************
 *
 * Private Slots
 *
 ***************************************************************************/

void PictureWidgetImpl::slotHueChanged(int hue)
{
    _qtv->setHue(hue);
} // slotHueChanged

void PictureWidgetImpl::slotColourChanged(int colour)
{
    _qtv->setColour(colour);
} // slotColourChanged

void PictureWidgetImpl::slotBrightnessChanged(int brightness)
{
    _qtv->setBrightness(brightness);
} // slotBrightnessChanged

void PictureWidgetImpl::slotWhitenessChanged(int whiteness)
{
    _qtv->setWhiteness(whiteness);
} // slotWhitenessChanged

void PictureWidgetImpl::slotContrastChanged(int contrast)
{
    _qtv->setContrast(contrast);
} // slotContrastChanged

#include "picturewidgetimpl.moc"
