/***************************************************************************
                          pluginwidgetimpl.cpp  -  description
                             -------------------
    begin                : Sat Jun 8 2002
    copyright            : (C) 2002 by Kevin Hessels <khessels@shaw.ca
                           (C) 2002 George Staikos <staikos@kde.org>
                           (C) 2003 Dirk Ziegelmeier <dziegel@gmx.de>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <klistview.h>
#include <klocale.h>
#include <kdebug.h>
#include <kconfig.h>
#include <kpushbutton.h>

#include "qtvision.h"
#include "pluginfactory.h"
#include "pluginwidgetimpl.h"
#include "osdmanager.h"
#include "miscmanager.h"
#include "qvosdplugin.h"
#include "qvmiscplugin.h"
#include "pluginconfigwidget.h"


class PluginListItem : public QCheckListItem
{
public:
    PluginListItem(KListView*, PluginDesc*);
    virtual ~PluginListItem();
    
    PluginDesc *pd;
};

PluginListItem::PluginListItem(KListView *parent, PluginDesc *d)
    :QCheckListItem(parent, d->name, QCheckListItem::CheckBox)
{
    setText(1, d->author);
    setText(2, d->comment);
    pd = d;
    setOn(pd->enabled);
}

PluginListItem::~PluginListItem()
{
}


/***************************************************************************/


PluginWidgetImpl::PluginWidgetImpl(QWidget* parent, QtVision* qtv, ConfigData* cfg, KConfig* h) 
    : PluginWidget(parent),
      SettingsDialogPage(i18n("Plugins"),
			 i18n("Configure plugins here"),
			 "energy"),
      _qtv(qtv),
      _cfg(cfg),
      _h(h)
{
    connect(miscListView, SIGNAL(selectionChanged()), 
	    this, SLOT(slotSelectionChanged()));
    connect(miscListView, SIGNAL(doubleClicked(QListViewItem *)),
	    this, SLOT(slotConfigurePlugin()));
    connect(_configure, SIGNAL(clicked()),
	    this, SLOT(slotConfigurePlugin()));

    _configure->setEnabled(false);
} // PluginWidgetImpl constructor


PluginWidgetImpl::~PluginWidgetImpl()
{
} // PluginWidgetImpl destructor


void PluginWidgetImpl::setup()
{
    miscListView->clear();

    for (PluginDesc *plug = _qtv->pluginFactory()->osdPlugins().first();
	 plug;
	 plug = _qtv->pluginFactory()->osdPlugins().next()) {
	new PluginListItem(miscListView, plug);
    } // for

    for (PluginDesc *plug = _qtv->pluginFactory()->miscPlugins().first();
	 plug;
	 plug = _qtv->pluginFactory()->miscPlugins().next()) {
        new PluginListItem(miscListView, plug);
    } // for
}

void PluginWidgetImpl::apply()
{
    _h->setGroup("Misc Plugins");

    bool rescanNecessary = false;
    QListViewItemIterator it(miscListView);
    for ( ; it.current(); ++it) {
        PluginListItem *item = static_cast<PluginListItem *>(it.current());
	if (item->pd->enabled != item->isOn()) {
	    rescanNecessary = true;
	}
        item->pd->enabled = item->isOn();
        _h->writeEntry(item->pd->name+"-"+item->pd->author, item->pd->enabled);
    } // for

    if (rescanNecessary) {
	_qtv->miscManager()->rescanPlugins();
	_qtv->osdManager()->rescanPlugins();
    }
} // applyPlugins

void PluginWidgetImpl::defaults()
{
}

/***************************************************************************
 *
 * Private slots
 *
 ***************************************************************************/


void PluginWidgetImpl::slotConfigurePlugin()
{
    PluginListItem *pli = static_cast<PluginListItem*>(miscListView->selectedItem());
    if (!pli || !pli->pd->configurable)
	return;
    PluginDesc* pd = pli->pd;
    
    QVPluginBase* p;
    bool previousState = pd->enabled;
    pd->enabled        = true;
    if (pd->type == PluginDesc::MISC) {
	p = static_cast<QVPluginBase*>(_qtv->pluginFactory()->getMiscPlugin(pd, _qtv->screen()));
    } else {
	p = static_cast<QVPluginBase*>(_qtv->pluginFactory()->getOSDPlugin(pd, _qtv->screen()));
    }
    pd->enabled        = previousState;

    PluginConfigWidget w(0L, "Plugin Configuration Dialog", true);
    w.setConfigWidget(i18n(pd->name), p->configWidget(&w, "Misc Configuration Widget"));
    if (w.exec() == QDialog::Accepted) {
	p->saveConfig();
    }

    p->destroy();
} // slotConfigurePlugin

void PluginWidgetImpl::slotSelectionChanged()
{
    PluginListItem *item = static_cast<PluginListItem *>(miscListView->selectedItem());
    if ( !item ) {
	_configure->setEnabled( false );
	return;
    }
    _configure->setEnabled(item->pd->configurable);
} // slotListItemClicked

#include "pluginwidgetimpl.moc"
