/*
 *
 * Copyright (C) 2002 George Staikos <staikos@kde.org>
 * Copyright (C) 2002 Richard Moore <rich@kde.org>
 *               2003 Dirk Ziegelmeier <dziegel@gmx.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <qcolor.h>

#include <kdebug.h>
#include <kcmenumngr.h>

#include "qtvision.h"
#include "viewmanager.h"

#include "qtvisionwidget.h"
#include "qtvisionwidget.moc"


QtVisionWidget::QtVisionWidget( QWidget *parent, const char *name, int wflags )
    : QHBox( parent, name ? name : "qtvision_widget", wflags ? wflags : WStyle_Customize | WStyle_Tool ),
      qtv(0), screen(0)
{
    setPaletteBackgroundColor( QColor( 0, 0, 0 ) );
    setDriver( new QtVision( this ) );
}

QtVisionWidget::QtVisionWidget( QtVision *qtv, QWidget *parent, const char *name, int wflags )
    : QHBox( parent, name ? name : "qtvision_widget", wflags ? wflags : WStyle_Customize | WStyle_Tool ),
      qtv(0), screen(0)
{
    setPaletteBackgroundColor( QColor( 0, 0, 0 ) );
    setDriver( qtv );
}

QtVisionWidget::~QtVisionWidget()
{
}

DCOPRef QtVisionWidget::driverIface() const
{
    return DCOPRef( qtv );
}

void QtVisionWidget::setDriver( QtVision *drv )
{
    if ( qtv && (qtv->parent() == this) ) {
	delete qtv;
	screen = 0;
    }

    qtv = drv;
    if ( !qtv )
	return;

    screen = qtv->createScreen( this );
}

void QtVisionWidget::addContextMenu( QPopupMenu *pop )
{
    if ( !pop )
	return;
    if ( !screen )
	return;

    KContextMenuManager::insert( screen, pop );
}
