// -*- c++ -*-

/*
 *
 * Copyright (C) 2002 Richard Moore <rich@kde.org>
 *               2003 Dirk Ziegelmeier <dziegel@gmx.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef QTVISIONWIDGET_H
#define QTVISIONWIDGET_H

#include <qhbox.h>

#include "qtvision.h"

class QPopupMenu;

/**
 * A widget that displays a television picture.
 *
 * @author Richard Moore, rich@kde.org
 */

class QtVisionWidget : public QHBox
{
    Q_OBJECT

public:
    /**
     * Creates a TV widget.
     */
    QtVisionWidget( QWidget *parent=0, const char *name=0, int wflags=0 );

    /**
     * Creates a TV widget with a specified driver.
     */
    QtVisionWidget( QtVision *qtv, QWidget *parent=0, const char *name=0, int wflags=0 );

    /**
     * Cleans up.
     */
    virtual ~QtVisionWidget();

    /**
     * Returns the QtVision object driving the TV picture.
     */
    QtVision *driver() const { return qtv; }

    /**
     * Returns the internal screen widget.
     */
    QtVisionView *view() const { return screen; }

    /**
     * Returns a DCOP reference to the driver object.
     */
    DCOPRef driverIface() const;

    /**
     * Returns the current channel store.
     */
    ChannelStore *channels() const { return (qtv ? qtv->channels() : 0); }

    /**
     * Returns the current view manager.
     */
    ViewManager *views() const { return (qtv ? qtv->views() : 0); }

    /**
     * Adds the specified context menu to the screen widget.
     *
     * @see KContextMenuManager
     */
    void addContextMenu( QPopupMenu *pop );


public slots:
    /**
     * Sets the QtVision object that drives the TV picture.
     */
    void setDriver( QtVision *qtv );


private:
    QtVision *qtv;
    QtVisionView *screen;
};

#endif // QTVISIONWIDGET_H
