/***************************************************************************
                           soundwidgetimpl.cpp
                           -------------------
    begin                : Sun Sep 6 2003
    copyright            : (C) 2003 by Dirk Ziegelmeier
    email                : dziegel@gmx.de
    rcs data             : $Id: soundwidgetimpl.cpp,v 1.8 2003/10/23 09:06:56 dziegel Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <klocale.h>
#include <kconfig.h>

#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qcheckbox.h>

#include "soundwidgetimpl.h"
#include "cfgdata.h"
#include "qvmixerplugin.h"
#include "qtvision.h"
#include "audiomanager.h"
#include "pluginconfigwidget.h"


SoundWidgetImpl::SoundWidgetImpl(QWidget* parent, QtVision* qtv, ConfigData* cfg, KConfig* h)
    : SoundWidget(parent),
      SettingsDialogPage(i18n("Sound"),
			 i18n("Configure sound options here"),
			 "kcmsound"),
      _qtv(qtv),
      _cfg(cfg),
      _h(h)
{
    connect(_mixer, SIGNAL(highlighted(const QString&)),
	    this, SLOT(mixerSelectionChanged(const QString&)));
    connect(_configure, SIGNAL(clicked()),
	    this, SLOT(configurePlugin()));
}

SoundWidgetImpl::~SoundWidgetImpl()
{
}

void SoundWidgetImpl::setup()
{
    _mixer->clear();
    for (PluginDesc *p = _qtv->pluginFactory()->mixerPlugins().first();
	 p;
	 p = _qtv->pluginFactory()->mixerPlugins().next())
    {
	_mixer->insertItem(p->name);
    }
    for (int i=0; i<_mixer->count(); i++) {
	if (_qtv->audioManager()->plugin()->pluginDescription() == getPluginDesc(_mixer->text(i))) {
	    _mixer->setCurrentItem(i);
	    break;
	}
    }
    mixerSelectionChanged(_mixer->currentText());

    _volumeIncrement->setValue(_cfg->volumeIncrement);
    _delayOnChange->setValue(_cfg->volumeRestoreDelay);
    _autoMute->setChecked(_cfg->autoMute);

    _prevMixer = _mixer->currentText();
}

void SoundWidgetImpl::defaults()
{
    _volumeIncrement->setValue(5);
    _delayOnChange->setValue(30);
    _autoMute->setChecked(false);
}

void SoundWidgetImpl::mixerSelectionChanged(const QString& mixer)
{
    PluginDesc *p = getPluginDesc(mixer);
    if (!p) return;

    _author->setText(p->author);
    _description->setText(p->comment);
    _configure->setEnabled(p->configurable);
}

PluginDesc* SoundWidgetImpl::getPluginDesc(const QString& mixer)
{
    for (PluginDesc *p = _qtv->pluginFactory()->mixerPlugins().first();
	 p;
	 p = _qtv->pluginFactory()->mixerPlugins().next())
    {
	if (p->name == mixer) return p;
    }
    return 0L;
}

void SoundWidgetImpl::configurePlugin()
{
    PluginDesc*    pd  = getPluginDesc(_mixer->currentText());
    if (!pd || !pd->configurable) return;

    bool previousState = pd->enabled;
    pd->enabled        = true;
    QVMixerPlugin* p   = _qtv->pluginFactory()->getMixerPlugin(pd);
    pd->enabled        = previousState;

    PluginConfigWidget w(0L, "Mixer Plugin Config Dialog", true);
    w.setConfigWidget(i18n(pd->name), p->configWidget(&w, "Mixer Configuration Widget"));
    if (w.exec() == QDialog::Accepted) {
	p->saveConfig();
    }

    p->destroy();
}

void SoundWidgetImpl::apply()
{
    _cfg->autoMute           = _autoMute->isChecked();
    _cfg->volumeRestoreDelay = _delayOnChange->value();
    _cfg->volumeIncrement    = _volumeIncrement->value();

    _h->setGroup("Mixer Plugins");
    for (PluginDesc* p = _qtv->pluginFactory()->mixerPlugins().first();
	 p;
	 p = _qtv->pluginFactory()->mixerPlugins().next())
    {
	if (p->name == _mixer->currentText()) {
	    _h->writeEntry(p->name + "-" + p->author, true);
	    p->enabled = true;
	} else {
	    _h->writeEntry(p->name + "-" + p->author, false);
	    p->enabled = false;
	}
    }

    if (_prevMixer != _mixer->currentText()) {
	_qtv->audioManager()->rescanPlugins();
    }
}

#include "soundwidgetimpl.moc"
