/***************************************************************************
                           vbidecoder.cpp
                           --------------
    begin                : Sun Oct 26 2003
    copyright            : (C) 2003 by Dirk Ziegelmeier
    email                : dziegel@gmx.de
    rcs data             : $Id: vbidecoder.cpp,v 1.1 2003/10/26 11:42:48 dziegel Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "vbidecoder.h"

#ifdef HAVE_ZVBI
#include <errno.h>
#include <unistd.h>

#include <kdebug.h>
#include <klocale.h>

VbiDecoder::VbiDecoder()
    : QThread(),
      _stationName(QString::null),
      _dec(0L),
      _cap(0L),
      _sliced(0L)
{
}

VbiDecoder::~VbiDecoder()
{
    stop();
}

bool VbiDecoder::init(const QString& dev)
{
    uint services = VBI_SLICED_VBI_525 | VBI_SLICED_VBI_625
        | VBI_SLICED_TELETEXT_B | VBI_SLICED_CAPTION_525
        | VBI_SLICED_CAPTION_625 | VBI_SLICED_VPS
        | VBI_SLICED_WSS_625 | VBI_SLICED_WSS_CPR1204;

    _cap = vbi_capture_v4l2_new(dev.local8Bit(), 16, &services, -1, 0, true);
    if (!_cap) {
	_cap = vbi_capture_v4l_new(dev.local8Bit(), 16, &services, -1, 0, true);
	if (!_cap) return false;
    }
    _dec = vbi_decoder_new();
    if (!_dec) {
	vbi_capture_delete(_cap);
	return false;
    }
    _par    = vbi_capture_parameters(_cap);
    _sliced = new vbi_sliced[_par->count[0] + _par->count[1]];
    _terminate = false;
    start();
    return true;
}

void VbiDecoder::stop()
{
    _terminate = true;
    wait();

    if (_sliced) delete[](_sliced);
    _sliced = 0L;
    if (_dec) vbi_decoder_delete(_dec);
    _dec = 0L;
    if (_cap) vbi_capture_delete(_cap);
    _cap = 0L;
}

static void vbi_decoder_vbi_event(vbi_event* event, void* data)
{
    reinterpret_cast<VbiDecoder*>(data)->vbiEvent(event);
}

void VbiDecoder::run()
{
    if (!_cap || !_dec || !_sliced) return;
    kdDebug() << "VbiDecoder: Running." << endl;
    vbi_event_handler_register(_dec, VBI_EVENT_NETWORK, &vbi_decoder_vbi_event, this);

    double ts = 0.0;
    int lines;
    timeval tv = { 1, 0 };

    while ( !_terminate ) {
	switch ( vbi_capture_read_sliced(_cap, _sliced, &lines, &ts, &tv ) ) {
	case -1:
	    kdDebug() << "VbiDecoder: VBI Error: " << strerror(errno) << endl;
	    _terminate = true;
	    break;
	case 1:
	    vbi_decode(_dec, _sliced, lines, ts);
	    break;
	default:
	    break;
	}
    }

    vbi_event_handler_unregister(_dec, &vbi_decoder_vbi_event, this);
    kdDebug() << "VbiDecoder: Stopped." << endl;
}

int VbiDecoder::vbiHandle()
{
    if (_cap) {
	return vbi_capture_fd(_cap);
    } else {
	return -1;
    }
}

void VbiDecoder::vbiEvent(vbi_event* event)
{
    QMutexLocker locker(&_eventMutex);
    _stationName.setLatin1((const char*)event->ev.network.name);
}

QString VbiDecoder::stationName()
{
    usleep(3000000);
    QMutexLocker locker(&_eventMutex);
    return QString(_stationName);
}
#endif
