/***************************************************************************
                           videowidgetimpl.cpp
                           -------------------
    begin                : Sun Sep 6 2003
    copyright            : (C) 2003 by Dirk Ziegelmeier
    email                : dziegel@gmx.de
    rcs data             : $Id: videowidgetimpl.cpp,v 1.7 2003/10/22 10:32:31 dziegel Exp $
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <klocale.h>
#include <klistview.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <kconfig.h>
#include <qspinbox.h>
#include <qcheckbox.h>

#include "qtvision.h"
#include "qvsrcplugin.h"
#include "videowidgetimpl.h"
#include "pluginconfigwidget.h"


class VideoPluginListItem : public QCheckListItem
{
public:
    VideoPluginListItem(KListView*, PluginDesc*);
    virtual ~VideoPluginListItem();
    
    PluginDesc* pd;
};

VideoPluginListItem::VideoPluginListItem(KListView *parent, PluginDesc *d)
    : QCheckListItem(parent, d->name, QCheckListItem::CheckBox)
{
    setText(1, d->author);
    setText(2, d->comment);
    pd = d;
    setOn(pd->enabled);
}

VideoPluginListItem::~VideoPluginListItem()
{
}



VideoWidgetImpl::VideoWidgetImpl(QWidget* parent, QtVision* qtv, ConfigData* cfg, KConfig* h)
    : VideoWidget(parent),
      SettingsDialogPage(i18n("Video"),
			 i18n("Configure video options here"),
			 "tv"),
      _qtv(qtv),
      _cfg(cfg),
      _h(h)
{
}

VideoWidgetImpl::~VideoWidgetImpl()
{
}

void VideoWidgetImpl::setup()
{
    _videoListView->clear();
    for (PluginDesc *p = _qtv->pluginFactory()->videoPlugins().first();
	 p;
	 p = _qtv->pluginFactory()->videoPlugins().next())
    {
	_videoListView->insertItem(new VideoPluginListItem(_videoListView, p));
    }
    selectionChanged();

    connect(_configure, SIGNAL(clicked()),
	    this, SLOT(configurePlugin()));
    connect(_videoListView, SIGNAL(doubleClicked(QListViewItem*)),
	    this, SLOT(itemDoubleClicked(QListViewItem*)));
    connect(_videoListView, SIGNAL(selectionChanged()),
	    this, SLOT(selectionChanged()));

    _fixAR->setChecked(_cfg->fixAR);
    _aspectCombo->setCurrentItem(_cfg->ARmode);

    _configure->setEnabled(false);
}

void VideoWidgetImpl::defaults()
{
    _fixAR->setChecked(false);
    _aspectCombo->setCurrentItem(AR_HEIGHT_TO_WIDTH);
}

void VideoWidgetImpl::selectionChanged()
{
    QListViewItem* item = _videoListView->selectedItem();

    if ( !item ) {
	_configure->setEnabled(false);
	return;
    }

    PluginDesc* p = static_cast<VideoPluginListItem*>(item)->pd;
    _configure->setEnabled(p->configurable);
}

void VideoWidgetImpl::itemDoubleClicked(QListViewItem*)
{
    configurePlugin();
}

void VideoWidgetImpl::configurePlugin()
{
    VideoPluginListItem* i = static_cast<VideoPluginListItem*>(_videoListView->selectedItem());
    if (!i) return;
    PluginDesc* pd         = i->pd;
    if (!pd->configurable) return;

    bool previousState = pd->enabled;
    pd->enabled        = true;
    QVSourcePlugin* p  = _qtv->pluginFactory()->getSourcePlugin(pd, _qtv->screen());
    pd->enabled        = previousState;
    p->probeDevices();

    PluginConfigWidget w(0L, "Video Plugin Config Dialog", true);
    w.setConfigWidget(i18n(pd->name), p->configWidget(&w, "Video Source Configuration Widget"));
    if (w.exec() == QDialog::Accepted) {
	p->saveConfig();
    }

    p->destroy();
}

void VideoWidgetImpl::apply()
{
    bool rescanNecessary = false;

    _cfg->fixAR     = _fixAR->isChecked();
    _cfg->ARmode    = _aspectCombo->currentItem();

    _h->setGroup("Video Plugins");
    for (QListViewItemIterator it(_videoListView);
	 it.current();
	 it++) {
	VideoPluginListItem* item = static_cast<VideoPluginListItem*>(it.current());
	if (item->pd->enabled != item->isOn()) {
	    rescanNecessary = true;
	}
        item->pd->enabled = item->isOn();
	_h->writeEntry(item->pd->name + "-" + item->pd->author, item->pd->enabled);
    }

    if (rescanNecessary) {
	_qtv->rescanSourcePlugins();
    }
}

#include "videowidgetimpl.moc"
